/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File debug.c - A primative Z80 debugger.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#ifdef _POSIX_SOURCE
  #include <termios.h>
  #include <sys/uio.h>
#endif

#include "z80.h"
#include "debug.h"

void help_message(void);
char waitforchar(void);
char input_char(void);
int disassemble(char *str, uns16 pc);

int 
run_debug(uns16 ThePC)
{		
		int TheChar;
		char sbuf[100];

		static int FirstTime = 1;
			
		if (FirstTime) {
			printf("SimCoupe Z80 debugger, type h for help\n\n");
			FirstTime = !FirstTime;
		}	
			
			
		disassemble(sbuf, ThePC);
		printf("%s\n",sbuf);
		TheChar = waitforchar();
		switch (TheChar) {
			case 'd':
				printf("\n");
/*				dump_cpu_state();*/
				return(2);
				break;
			case 'r':
				printf("Returning to Emulation mode\n\n");
				fflush(stdout);
				return(0);
				break;
			case 'z':
				{ 
				 unsigned int pc_tmp;
				 printf("Disassembly, type z to return\n");
				 printf("address (in Hex) : ");
				 scanf("%x",&pc_tmp);
				 do { 
				   pc_tmp = disassemble(sbuf,pc_tmp);
				   printf("Disassembly : %s\n",sbuf);
				 } while (waitforchar() !='z');
				}
				break;
			case 'h' :
				help_message();
				break;	
			default:
				break;
		} /* End case */									        			

		return(1);
}

void 
help_message()
{
	printf("\n");
	printf("SimCoupe Z80 debugger - (c) 1996 A.J.Skillman\n");
	printf("-------------------------------------------\n");
	printf("\nCommands :\n\t\td : Register Dump\n"
			     "\t\th : Show this help\n"
			     "\t\tr : Exit Debugger\n"
			     "\t\tz : Show Disassembly\n\n");
	printf("-------------------------------------------\n\n");
			    
}

char
waitforchar()
{
	char tmp[255], ret;
	
	fflush(stdin);
	ret = input_char();

	return(ret);
}

char input_char(void)
/************************************************************/
/* Get single character without waiting for <return>        */
/************************************************************/
{
   char buffer[4];
   int inlen;

#ifdef _POSIX_SOURCE
   int status ;
   struct termios my_termios ;

   status = tcgetattr (0, &my_termios) ;
   if (status)
   {
      printf ("Error %d during tcgetattr\n", errno) ;
      exit (errno) ;
   }

   /* Switch off canonical input -- allow single char reads */
   my_termios.c_lflag &= ~(ICANON | ECHO);

   tcsetattr (0, TCSANOW, &my_termios) ;
#endif

   fflush (stdout);
   inlen = read (0, buffer, 1);

#ifdef _POSIX_SOURCE
   /* Switch on canonical input -- allow character cooking */
   my_termios.c_lflag |= ICANON | ECHOE | ECHO;

   tcsetattr (0, TCSANOW, &my_termios) ;
#endif

   return ((inlen > 0) ? (buffer[0]) : '\n');
}

