/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File fdi.h public interface to the floppy disk interface emulation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _FDI_H
#define _FDI_H

#define MAX_NO_SIDES	2
#define MAX_NO_TRACKS	80
#define MAX_NO_SECTORS	10

#define MAX_SECTOR_SIZE	5120

/* Disk image types */

#define SAM_IMAGE	0
#define MSDOS_IMAGE	1
#define SAD_IMAGE	2
#define REAL_FD		3

/* The various disk format image sizes */

#define SAM_FORMAT_SIZE		819200
#define MSDOS_FORMAT_SIZE	737280

/* The ID string for Aley Keprt's SAD disk image format */
#define SAD_FORMAT_ID	"Aley's disk backup"

/* The size (in bytes) of the SAD image header */
#define SAD_HEADER		22

/* Special path names for real floppy drives */

#define FDI_FLOPPY_1	"fd1:"
#define FDI_FLOPPY_2	"fd2:"


/* Default DOS system image */
#define SYSTEM_DOS      "system.dos"

/* The VL1772 commands */
#define RESTORE		    0x00
#define SEEK		    0x10
#define STEP_NUPD	    0x20
#define STEP_UPD	    0x30
#define STEP_IN_NUPD	0x40
#define STEP_IN_UPD	    0x50
#define STEP_OUT_NUPD	0x60
#define STEP_OUT_UPD	0x70
#define READ_1SECTOR	0x80
#define READ_MSECTOR	0x90
#define WRITE_1SECTOR	0xa0
#define WRITE_MSECTOR	0xb0
#define READ_ADDRESS	0xc0
#define READ_TRACK	    0xe0
#define WRITE_TRACK	    0xf0
#define FORCE_INTERRUPT	0xd0

/* The possible status bits */
#define BUSY	    		0x01
#define DRQ		    	    0x02
#define LOST_DATA		    0x04
#define CRC_ERROR	    	0x08
#define RECORD_NOT_FOUND	0x10
#define SPIN_UP			    0x20
#define WRITE_PROTECT		0x40
#define MOTOR_ON	    	0x80


void fdi_out(uns16 port, uns8 byte);
uns8 fdi_in (uns16 port);

void insert_disk(char *fn, int disk_no);
void eject_disk(int disk_no);

int create_disk_image(char *ImageFile, int AddSystem);
#endif
