
/*
 * Source file joystick.c - emulate a SAM joystick.
 * Only works on Linux with joystick kernel patches, tested
 * with versions 0.5, 0.6 & 0.7.
 */

#ifdef JOY
#include <fcntl.h>
#include <stdio.h>
#include <linux/joystick.h>

#include "resource.h"
#include "util.h"

/* joystick bits */
#define JOYST_RIGHT 0x08
#define JOYST_LEFT 0x10
#define JOYST_DOWN 0x04
#define JOYST_UP 0x02
#define JOYST_FIRE 0x1

static int jsfd;	/* joystick file descriptor */
static int joystick_ok = 0;

static int centre_x, centre_y;

int
joystick_init(void)
{
	struct JS_DATA_TYPE js_data;
	long tmpl;
	int tmp;

	if ((jsfd = open(GetCfg(js_device), O_RDONLY)) < 1) {
		simcoupe_mesg(SIMCOUPE_WARN, "joystick initialisation failure!");
		perror("  -> reason: open");
		return -1;
	}

	tmpl = 10; 

	if (ioctl(jsfd, JS_SET_TIMELIMIT, &tmpl) == -1) {
		simcoupe_mesg(SIMCOUPE_WARN, "joystick initialisation failure!");
		perror("  -> reason: ioctl JS_SET_TIMELIMIT");
		return -1;
	}

	simcoupe_mesg(SIMCOUPE_INFO, "Linux joystick option selected:");
	simcoupe_mesg(SIMCOUPE_INFO, "  Move joystick to lower right and press either button");

	while ((read(jsfd, &js_data, JS_RETURN) > 0) && js_data.buttons == 0x00)
		;

	for(tmp=0; js_data.x > 0xff; tmp++, js_data.x >>= 1)
		;
	js_data.x=tmp;
	for(tmp=0; js_data.y > 0xff; tmp++, js_data.y >>= 1)
		;
	js_data.y=tmp;

	if (ioctl(jsfd, JS_SET_CAL, &js_data) == -1) {
		simcoupe_mesg(SIMCOUPE_WARN, "joystick calibration failure!");
		perror("  -> reason: ioctl JS_SET_CAL");
		return -1;
	}

	while ((read(jsfd, &js_data, JS_RETURN) > 0) && js_data.buttons != 0x00)
		;
	simcoupe_mesg(SIMCOUPE_INFO, "  Now centralise the joystick and press either button");
	while ((read(jsfd, &js_data, JS_RETURN) > 0) && js_data.buttons == 0x00)
		;
	centre_x = js_data.x; centre_y = js_data.y;

#ifdef DEBUG
	simcoupe_mesg(SIMCOUPE_INFO, "SimCoupe: joystick initialisation successful\n");
#endif

	joystick_ok = 1;
	return 0;
}


/* Read the current joystick position.  If it's displaced by more than
 * <js_tolerance> from centre, then OR in the appropriate direction bit.
 */
int 
joystick_read(void)
{
	struct JS_DATA_TYPE js_data;
	int res = 0;

	if (!joystick_ok)
		return 0;

	if (read(jsfd, &js_data, JS_RETURN) != JS_RETURN) {
		simcoupe_mesg(SIMCOUPE_WARN, "lost joystick connection!");
		joystick_ok = 0;
		return 0;
	}

	/* decode the joystick return values */
	if (js_data.x < centre_x - GetCfg(js_tolerance)) res |= JOYST_LEFT;
	else if (js_data.x > centre_x + GetCfg(js_tolerance)) res |= JOYST_RIGHT;

	if (js_data.y < centre_y - GetCfg(js_tolerance)) res |= JOYST_UP;
	else if (js_data.y > centre_y + GetCfg(js_tolerance)) res |= JOYST_DOWN;

	if (js_data.buttons) res |= JOYST_FIRE;

	return res;
}
#endif
