/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File mem.c - where memory is actually held.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * NOTES
 * -----
 * The SAM memory is implemented in NMAINPAGES 16K pages and pointed to 
 * by the array *realMem[]. (The actual space is allocated at run time)
 * The Z80 Address space is asigned using an array of 4 pointers to this
 * space. The page number asigned to Sections A..D of the address space
 * is stored in real_pagenos[].
 * All memory addressing in the Z80 emulation is done indirectly via these
 * pointers - see file mem.h
 */

#include <stdlib.h>
#include <stdio.h>

#include "z80.h"
#include "mem.h"
#include "util.h"
#include "resource.h"


uns8 *theRdMemory[4];		/* 4 pointers to real memory */
uns8 *theWrMemory[4];		/* 4 pointers to real memory */
int real_pagenos[4];		/* index to real memory page */
uns8 *realMem[TOTAL_NO_PAGES];	/* real memory is kept here */
uns8 *scratchMem[4];            /* scratch memory for read only areas */

/* Allocate NMAINPAGES chunks of main memory, and the required
   number of external memory pages, each of size SAM_PAGESIZE */
void
init_memory(void)
{
	static int inited = 0;
	int i, iMeg, NoMbs;

	if (inited) {
		simcoupe_mesg(SIMCOUPE_ERR, "attempt to re-initialise memory (failed)");
		return;
	}

	/* initialise the main SAM memory */
	for (i = 0; i < NMAINPAGES; i++) {
		if ((realMem[i] = (uns8 *)malloc(SAM_PAGESIZE)) == (uns8 *)0)
			simcoupe_mesg(SIMCOUPE_FATAL, "can't allocate 16K for page %d!", i);
	}

	/* initialise the memory for the 2 roms */
	for (i = 0; i < 2; i++) {
		if ((realMem[ROM0+i] = (uns8 *)malloc(SAM_PAGESIZE)) == (uns8 *)0)
			simcoupe_mesg(SIMCOUPE_FATAL, "can't allocate 16K for ROM %d!", i);
	}

	/* initialise the memory for the 4 scratch pages */
	for (i = 0; i < 4; i++) {
		if ((scratchMem[i] = (uns8 *)malloc(SAM_PAGESIZE)) == (uns8 *)0)
			simcoupe_mesg(SIMCOUPE_FATAL, "can't allocate 16K for scratch area %d!", i);
	}

	/* initialise the external memory if required*/
	if (NoMbs = GetCfg(externalMem)) {

		if (NoMbs > MAXEXPAGES) NoMbs = MAXEXPAGES;
		for (iMeg = 0; iMeg < NoMbs; iMeg++) {
			for (i = 0; i < NEXPAGES; i++) {
		
				if ((realMem[NMAINPAGES+i+(3-iMeg)*NEXPAGES] = (uns8 *)malloc(SAM_PAGESIZE)) == (uns8 *)0)
					simcoupe_mesg(SIMCOUPE_FATAL, "can't allocate 16K for external page %d!", i+iMeg*NEXPAGES);
			}
		}
	}
}



/* Set up the initial memory pages; ROM0 paged in at 0x0000
 */
void
init_pages(void)
{
	page_in(0, ROM0, True ); 
	page_in(1, 1,    False);
	page_in(2, 2,    False); 
	page_in(3, 3,    False);
}
