/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File mem.h - definitions and inline memory i/o routines.
 *
 * Intel 80x86 assembly optimisations Copyright (c) 1996 Allan Skillman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _MEM_H
#define _MEM_H

#include <string.h>

#include "sam.h"
#include "resource.h"


#if !defined(__GNUC__)
#define inline
#endif

void init_pages(void);

/* 16K memory pages */
#define SAM_PAGESIZE 0x4000

/* Constants related to memory organisation */

#define NMAINPAGES	32	/* Number of pages in the SAM's main memory (512K) */
#define NEXPAGES	64	/* Number of pages in 1Mb of external memory */ 
#define MAXEXPAGES	4       /* Maximum number of 1Mb external memory allowed */

/* The Roms always go at the end of the memory */

#define ROM0   		NMAINPAGES+(NEXPAGES*MAXEXPAGES)
#define ROM1   		ROM0 + 1
#define TOTAL_NO_PAGES	ROM1 + 1

#define SCRATCH_PAGE	0xffff

extern uns8 *theRdMemory[4];
extern uns8 *theWrMemory[4];
extern int real_pagenos[4];
extern uns8 *realMem[TOTAL_NO_PAGES];
extern uns8 *scratchMem[4];


/* Map a 16 bit address through the memory indirection - allows fast paging */
#define VPAGE(addr) ((addr) >> 14)
#define RPAGE(addr) (real_pagenos[VPAGE(addr)])
#define MEM_R(addr) (theRdMemory[VPAGE(addr)][(addr) & 0x3fff])
#define MEM_W(addr) (theWrMemory[VPAGE(addr)][(addr) & 0x3fff])
#define READ_ONLY(addr) ((RPAGE(addr) >= ROM0) || (!((addr) & 0xc000) && LMPR_WPRAM))

void init_memory(void);

/*
 * Inlined memory acess functions, includes intel x86 assembly versions
 * selected using the compile time macro INTEL_ASM
 */
static inline uns8
read_byte(uns16 addr)
{
#ifndef INTEL_ASM
	return MEM_R(addr);
#else
	{ register uns8 rtn; 	
	  asm("movzwl %%ax,%%ebx; shrw $14,%%bx; movl _theRdMemory(,%%ebx,4),%%ebx\n"
	      "andl $0x3fff,%%eax; movzbw (%%ebx,%%eax), %%ax"
	      : "a=" (rtn)
	      : "a"  (addr)
	      : "eax", "ebx");
	  return rtn;       
	}
#endif
}

static inline uns16
read_word(uns16 addr)
{
#ifndef INTEL_ASM
	return ((MEM_R(addr)) + (MEM_R((uns16)(addr + 1)) << 8));
#else
	{ register uns16 rtn; 	
	  asm("movzwl %%ax,%%ebx; shrw $14,%%bx; movl _theRdMemory(,%%ebx,4),%%ecx\n"
	      "andl $0x3fff,%%eax; cmpl $0x3fff,%%eax; jne 0f\n"
	      "movb (%%ecx,%%eax), %%al; incb %%bl; andb $0x03,%%bl\n"
	      "movl _theRdMemory(,%%ebx,4),%%ecx; movb (%%ecx,%%eax), %%ah\n"
	      "jmp 1f\n" 
	      "0: movw (%%ecx,%%eax), %%ax; 1:"
	      : "a=" (rtn)
	      : "a"  (addr)
	      : "eax", "ebx", "ecx");
	  return rtn;       
	}


#endif
}


static void
write_to_screen_vmpr0(uns16 addr) 
{

	extern unsigned char dtbl_y[]; 

	/* from (vga/x)_samgrx.c */
	extern uns8 LineNeedsUpdating[];

	int AttrLine;

	addr &= 0x3fff;

        switch (VMPR_SCREEN_MODE) {
           
	        case MODE_1:      
	        
		   if (addr < 6144)
			LineNeedsUpdating[dtbl_y[addr]] = True;
		   else if (addr < 6912) {
		   	AttrLine = ((addr-6144) & 0xffe0)>>2;
			memset(&LineNeedsUpdating[AttrLine], True, 8);
		   }	
		   break;
	        case MODE_2:      
		   if (addr < 0x3800)
			LineNeedsUpdating[(addr & 0x1fff)>>5] = True;
		   break;
	        case MODE_3:
	        case MODE_4:      
		   LineNeedsUpdating[addr>>7] = True;
		   break;
	} /* End Switch */
	   
}  

static inline  void
write_to_screen_vmpr1(uns16 addr) {

	/* from (vga/x)_samgrx.c */
	extern uns8 LineNeedsUpdating[];

	addr &= 0x3fff;
	if (addr < 0x2000)
		LineNeedsUpdating[128+(addr>>7)] = True;        
}

static inline void 
write_byte(uns16 addr, uns8 val)
{

#ifndef INTEL_ASM
	MEM_W(addr) = val;
#else
	asm("movzwl %%ax,%%ebx; shrw $14,%%bx; movl _theWrMemory(,%%ebx,4),%%ebx\n"
      	    "andl $0x3fff,%%eax; movb %%cl, (%%ebx,%%eax)"
      	    : 
      	    : "a"  (addr), "c" (val)
      	    : "eax", "ebx", "cx");
#endif		
	/* Only Do a screen write if required */
	if (RPAGE(addr) == VMPR_SCREEN_PAGE)
 		write_to_screen_vmpr0(addr);
 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
	         (RPAGE(addr) == ((VMPR_SCREEN_PAGE + 1) & 0x1f)))
	    	write_to_screen_vmpr1(addr);
}




static void
write_word(uns16 addr, uns16 val)
{

#ifndef INTEL_ASM
	MEM_W(addr) = val & 0xff;
#else
	asm("movzwl %%ax,%%ebx; shrw $14,%%bx; movl _theWrMemory(,%%ebx,4),%%ebx\n"
      	    "andl $0x3fff,%%eax; movb %%cl, (%%ebx,%%eax)"
      	    : 
      	    : "a"  (addr), "c" (val)
      	    : "eax", "ebx", "cx");
#endif		
	/* Only Do a screen write if required */
	if (RPAGE(addr) == VMPR_SCREEN_PAGE)
 		write_to_screen_vmpr0(addr);
 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
	         (RPAGE(addr) == ((VMPR_SCREEN_PAGE + 1) & 0x1f)))
	    	write_to_screen_vmpr1(addr);

	addr++;

#ifndef INTEL_ASM
	MEM_W(addr) = (val & 0xff00) >> 8;
#else
	asm("movzwl %%ax,%%ebx; shrw $14,%%bx; movl _theWrMemory(,%%ebx,4),%%ebx\n"
      	    "andl $0x3fff,%%eax; movb %%ch, (%%ebx,%%eax)"
      	    : 
      	    : "a"  (addr), "c" (val)
      	    : "eax", "ebx", "cx");
#endif		
		
	/* Only Do a screen write if required */
	if (RPAGE(addr) == VMPR_SCREEN_PAGE)
	    	write_to_screen_vmpr0(addr);
 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
	         (RPAGE(addr) == ((VMPR_SCREEN_PAGE + 1) & 0x1f)))
	    	write_to_screen_vmpr1(addr);
		
}

/* Page in real memory page at <frame>, where <frame> is in range 0..3 */
static inline void
page_in(int frame, int rpage, int ReadOnly)
{

	theRdMemory[frame] = realMem[rpage];
    if (!ReadOnly) {
		theWrMemory[frame] = realMem[rpage];
		real_pagenos[frame] = rpage;
	}
	else {
		theWrMemory[frame]  = scratchMem[frame];
		real_pagenos[frame] = SCRATCH_PAGE;
	}
/*	printf("page_in(%d,%d)\n",frame,rpage); */
}


#endif
