/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File sam.h - public interface to sam hardware emulation code.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _SAM_H
#define _SAM_H

void init_display(void);
void init_ports(void);
void out_byte(uns16 port, uns8 val);
uns8 in_byte(uns16 port);
void check_events(void);
void ChangeScaleMode(int SF);
int init_buffer_mode(void);
int get_colours(void);
int image_init(void); 

void (*FlyBack)(void);
void (*border_change)(void);

/* a list of actions which can be bound to keypresses */
enum {
	KM_JOY_UP, KM_JOY_DOWN,
	KM_JOY_LEFT, KM_JOY_RIGHT,
	KM_JOY_FIRE,
#ifdef SLOWDOWN
	KM_KEY_FASTER, KM_KEY_SLOWER,
#endif
	NUM_KEYMAPPINGS
};

/* default dimentions - scaled accordingly in vgasam.c */

#ifdef VGA_DISPLAY
#define HSize1x1                320
#define HSize2x2                640
#define VSize1x1                200
#else
#define HSize1x1                304
#define HSize2x2                608
#define VSize1x1                240
#endif
#ifdef USE_VGA_TRICKS
#define VSize2x1		240
#endif
#define VSize2x2                480

#define BorderWidth_H_1x1       ((HSize1x1-256)/2)
#define BorderWidth_V_1x1       ((VSize1x1-192)/2)
#define BorderWidth_H_2x2       ((HSize2x2-512)/2)
#ifdef USE_VGA_TRICKS
#define BorderWidth_V_2x1       ((VSize2x1-192)/2)
#endif
#define BorderWidth_V_2x2       ((VSize2x2-384)/2)

/* Mouse device - should normally be /dev/mouse (only used for vga display) */

#define MOUSE_DEVICE    "/dev/mouse"

#ifdef PSEUDO_IO
#define PSEUDO_IO_LOWBYTE 0xbf
#define PSEUDO_STDIN  (0x0000 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDOUT (0x0100 | PSEUDO_IO_LOWBYTE)
#define PSEUDO_STDERR (0x0200 | PSEUDO_IO_LOWBYTE)
#endif

/* The Number of Colours the SAM has (128 of course) */
#define NCOLOURS 128

/* Macros for access to sam ports */

#define NO_SAA_REGS	 32

#define SOUND_ADDR      511
#define SOUND_DATA      255

#define VMPR_PORT       252
#define HMPR_PORT       251
#define LMPR_PORT       250
#define MIDI_PORT       253
#define BORDER_PORT     254
#define LINE_PORT       249
#define CLUT_BASE_PORT  248
#define ATTR_PORT       255
#define KEYBOARD_PORT   254
#define STATUS_PORT     249
#define LPEN_PORT       248
#define HPEN_PORT       504
#define CLOCK_PORT      239

#define LEPR_PORT       128
#define HEPR_PORT       129

#define DISK1_PORT      224
#define DISK2_PORT      240

#define PRINTL1_DATA    232
#define PRINTL1_STAT    233

#define PRINTL2_DATA    234
#define PRINTL2_STAT    235

#define VMPR_SCREEN_PAGE        (vmpr & 0x1f)
#define VMPR_SCREEN_MODE        (vmpr & 0x60)
#define HMPR_PAGE               (hmpr & 0x1f)
#define HMPR_MD3COL            ((hmpr & 0x60) >> 3)
#define HMPR_MCNTRL             (hmpr & 0x80)
#define LMPR_PAGE               (lmpr & 0x1f)
#define LMPR_RAM0               (lmpr & 0x20)
#define LMPR_ROM1               (lmpr & 0x40)
#define LMPR_WPRAM              (lmpr & 0x80)

#define BORD_VAL(x)            ((((x) & 0x20 ) >> 2) | ((x) & 0x07))
#define BORD_COL(x)            ((x) & 0x27)
#define BORD_SOFF               (border & 0x80)

#define MD3_SWITCH(x)          (((x) == 0x01 || (x) == 0x02) ? (x) ^ 0x03 : (x))

#define MODE_1                  0x00
#define MODE_2                  0x20
#define MODE_3                  0x40
#define MODE_4                  0x60

/* The sam ports */

/* Read / Write ports */

extern uns8 vmpr;
extern uns8 hmpr;
extern uns8 lmpr;
extern uns8 midi;
extern uns8 border;

extern uns8 lepr;
extern uns8 hepr;

extern uns8 clock_reg;

/* Write only ports */

extern uns8 line_int;
extern uns8 clut[];

/* Read only ports */

extern uns8 attr_reg;
extern uns8 keyboard;
extern uns8 status_reg;
extern uns8 lpen;
extern uns8 hpen;

#endif
