/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File samgrx.c : SAM display driver routines - DOS vga version
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>


#include "z80.h"
#include "mem.h"
#include "sam.h"
#include "util.h"
#include "keyb_int.h"
#include "video.h"

extern uns8 *vga_data_ptr;
extern int flash_phase;
extern int ClutChangeInFrame;
extern int BorderChangeInFrame;
extern int VmprChangeInFrame;   
extern int Vga2x2LinearAddressing;

extern int Video1x1Mode;
extern int Video1x1HasLinear;
extern int Video2x2Mode;
extern int Video2x2HasLinear;



/* Used for paged vga addressing for 2x2 scale mode */

#ifndef USE_VGA_TRICKS
static int SwitchOverLines[4] = { 27, 78, 129, 180 };
#else
static int SwitchOverLines[2] = { 78, 180 };
#endif
static int VgaPageNo;
static uns8 TmpBuff[1280];
static uns8 *TmpPtr = NULL; 
static uns8 *vga_data_end;
static int VgaMidPoint;

/* These macros map a byte in the attribute area to an vga colour index */
#define Background(byte) (((byte) & 0x38) >> 3) | (((byte) & 0x40) >> 3)
#define Foreground(byte) ((byte) & 0x07) | (((byte) & 0x40) >> 3)

uns8 LineNeedsUpdating[256];    /* Determines whether a given line needs to
				   be redrawn */

static uns8 *SamMemoryPtr;      /* Pointer to the SAM screen memory */
static uns8 *VgaMemoryPtr;      /* Pointer within the vga frame buffer */

/* General level function pointers */

void (*borderline_update)(void);
void (*line_update)(int);
void (*line_update_mode_1)(int);
void (*line_update_mode_2)(int);
void (*line_update_mode_3)(int);
void (*line_update_mode_4)(int);


/* Scale mode level function pointers */

void (*borderline_update_1x1)(void);
void (*borderline_update_2x2)(void);
void (*line_update_vga_mode1_1x1)(int);
void (*line_update_vga_mode2_1x1)(int);
void (*line_update_vga_mode3_1x1)(int);
void (*line_update_vga_mode4_1x1)(int);
void (*line_update_vga_mode1_2x2)(int);
void (*line_update_vga_mode2_2x2)(int);
void (*line_update_vga_mode3_2x2)(int);
void (*line_update_vga_mode4_2x2)(int);

int BorderTop;
int BorderBottom;

/* Pointers to internal video mode functions */

void (*PutLine)(char *addr, int target_y);
void (*ChangeBank)(int BankNumber);

/* Include the memory model specfic code */

#include "vgafbmap.c"
#include "vgafbcpy.c"

void
ChangeScaleMode(int SF)
{

	int i;
	
	if (SF == 1) {
	
		borderline_update      = borderline_update_1x1;
		line_update_mode_1 = line_update_vga_mode1_1x1;
		line_update_mode_2 = line_update_vga_mode2_1x1;
		line_update_mode_3 = line_update_vga_mode3_1x1;
		line_update_mode_4 = line_update_vga_mode4_1x1;

#ifdef USE_VGA_TRICKS
		VideoVgaTricks(VIDEO_NORMAL);
#endif
		VideoSetMode(Video1x1Mode);
		BorderTop    = 55; 
		BorderBottom = 255;

		PutLine    = VideoGetPutLine(Video1x1Mode);
		ChangeBank = VideoGetChangeBank(Video1x1Mode); 

#if 0
		VideoVgaTricks(VIDEO_STRETCH);
#endif
	}
	else {
	
		borderline_update      = borderline_update_2x2;
		line_update_mode_1 = line_update_vga_mode1_2x2;
		line_update_mode_2 = line_update_vga_mode2_2x2;
		line_update_mode_3 = line_update_vga_mode3_2x2;
		line_update_mode_4 = line_update_vga_mode4_2x2;

#ifdef USE_VGA_TRICKS
		VideoVgaTricks(VIDEO_NORMAL);
#endif
		VideoSetMode(Video2x2Mode);

		BorderTop    = 35; 
		BorderBottom = 286;
		PutLine    = VideoGetPutLine(Video2x2Mode);
		ChangeBank = VideoGetChangeBank(Video2x2Mode); 

#ifdef USE_VGA_TRICKS
		VideoVgaTricks(VIDEO_DOUBLE_SCAN);
#endif
	}
	vga_data_ptr = VideoGetFramePointer();
	vga_data_end = vga_data_ptr + 0x10000;
	get_colours();
	/* Make sure we update the whole screen */
	for (i=0; i < 192; i++)
		LineNeedsUpdating[i] = True;
	BorderChangeInFrame = (192+119);
	
	FlyBack();

}

int
init_buffer_mode(void)
{
	/* Initialise the display related function pointers to the */
	/* correct actual functions for a mapping mode		   */

	int buffer_is_mapped = VideoCanMapBuffer();

	if (buffer_is_mapped) {
		line_update_vga_mode1_1x1 = line_update_vga_mode1_1x1_mapped; 
		line_update_vga_mode2_1x1 = line_update_vga_mode2_1x1_mapped; 
		line_update_vga_mode3_1x1 = line_update_vga_mode3_1x1_mapped; 
		line_update_vga_mode4_1x1 = line_update_vga_mode4_1x1_mapped; 
		line_update_vga_mode1_2x2 = line_update_vga_mode1_2x2_mapped; 
		line_update_vga_mode2_2x2 = line_update_vga_mode2_2x2_mapped; 
		line_update_vga_mode3_2x2 = line_update_vga_mode3_2x2_mapped; 
		line_update_vga_mode4_2x2 = line_update_vga_mode4_2x2_mapped; 

		borderline_update_1x1   = borderline_update_1x1_mapped; 
		borderline_update_2x2   = borderline_update_2x2_mapped; 
		FlyBack = FlyBack_mapped;
		simcoupe_mesg(SIMCOUPE_INFO,"Using a mapped Frame Buffer");
	}
	else {
		line_update_vga_mode1_1x1 = line_update_vga_mode1_1x1_copy; 
		line_update_vga_mode2_1x1 = line_update_vga_mode2_1x1_copy; 
		line_update_vga_mode3_1x1 = line_update_vga_mode3_1x1_copy; 
		line_update_vga_mode4_1x1 = line_update_vga_mode4_1x1_copy; 
		line_update_vga_mode1_2x2 = line_update_vga_mode1_2x2_copy; 
		line_update_vga_mode2_2x2 = line_update_vga_mode2_2x2_copy; 
		line_update_vga_mode3_2x2 = line_update_vga_mode3_2x2_copy; 
		line_update_vga_mode4_2x2 = line_update_vga_mode4_2x2_copy; 

		borderline_update_1x1   = borderline_update_1x1_copy; 
		borderline_update_2x2   = borderline_update_2x2_copy; 
		FlyBack = FlyBack_copy;
		simcoupe_mesg(SIMCOUPE_INFO,"Using a unmapped Frame Buffer");
	}
	return True;
}
