/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File video.c - Misc video utilities for VESA interface 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pc.h>

#include "vbe.h"
#include "video.h"

#define COLOUR_MODE	T256

static char *PresentFramePointer = NULL;

int
VideoInit( void )
{
	return InitGfxLib();
}

void
VideoClose( void )
{
	CloseGfxLib();
}

int
VideoFindMode(int gfx_height, int gfx_width)
{
	int i;
	int ModeNumber = -1;

	int HasLinear = 0;

	if (InitGfxLib()) {
	 
	  /* Look for mode */

#if 0
	  /* Try for a linear mode first */

	  for (i = 0; i < NumberOfModes; i++) {
	    if ((ModeList[i].ModeType == COLOUR_MODE) &&
		(ModeList[i].ModeWidth == gfx_width) &&
		(ModeList[i].ModeHeight == gfx_height)) {
		if (ModeList[i].HasLinear) {
		    ModeNumber = i;
		    HasLinear = ModeList[i].HasLinear;
		    break;
		}
	    }
	  }

#endif
	  /* Now look for a non linear mode */
	  if (ModeNumber == -1 ) {
	   for (i = 0; i < NumberOfModes; i++) {
	    if ((ModeList[i].ModeType == COLOUR_MODE) &&
		(ModeList[i].ModeWidth == gfx_width) &&
		(ModeList[i].ModeHeight == gfx_height)) {
		if (ModeList[i].HasWindow || ((!ModeList[i].HasLinear) && (!ModeList[i].HasWindow))) {
		    ModeNumber = i;
		    HasLinear = ModeList[i].HasLinear;
		    break;
		}
	    }
	   }
	  }

	  CloseGfxLib();
	}
	return ModeNumber;

}

char *
VideoGetFramePointer(void)
{
	static char *PresentFramePointer = NULL;

	if (PresentFramePointer != NULL)
	    free(PresentFramePointer);

	/* Only Map the frame buffer if it is possible! */
	if (CanMapBuffer) {

		if (CurrentMode.HasLinear) {
			PresentFramePointer = MapVideoMemory(malloc(CurrentMode.LinearMappingSize));
			memset(PresentFramePointer, 0x00,(size_t)CurrentMode.LinearMappingSize); 
		}
		else {
			PresentFramePointer = MapVideoMemory(malloc(CurrentMode.WindowMappingSize));
			memset(PresentFramePointer, 0x00,(size_t)CurrentMode.WindowMappingSize); 
		}
	}

	return (PresentFramePointer);
}

void
VideoFreeFramePointer(void)
{
	if (PresentFramePointer != NULL)
		free(PresentFramePointer);

}

int
VideoCanMapBuffer(void)
{
	return CanMapBuffer;
}

int
VideoModeIsLinear(int TheVideoMode)
{
	return ModeList[TheVideoMode].HasLinear;
}

void
VideoSetMode(int TheVideoMode)
{
	SetMode(TheVideoMode, ModeList[TheVideoMode].HasLinear);
}

void *
VideoGetPutLine(int TheVideoMode)
{
	return ModeList[TheVideoMode].PutLine;
}

void *
VideoGetChangeBank(int TheVideoMode)
{
	return ModeList[TheVideoMode].ChangeBank;
}

void
VideoLoadPalette(void)
{
	LoadPalette();
}

void 
VideoDelayedSetPalette(int Color, int Red, int Green, int Blue)
{
	DelayedSetPalette(Color, Red, Green, Blue);
}

void
VideoVgaTricks(int VgaTrickID) 
{
	/* Perform various VGA tricks using direct Port IO */

	switch (VgaTrickID) {
		case VIDEO_DOUBLE_SCAN:
			/* Halve the number of Vert. scan lines */
			outportb(0x3d4, 9);
			outportb(0x3d5, inportb(0x3d5) | 0x80);
			break;
		case VIDEO_STRETCH:
			/* Stretch the vertical display size */
			outportb(0x3c2, inportb(0x3cc) | 0xc0);
			break;
		case VIDEO_NORMAL:
			/* Reset to normal VGA settings */
			outportb(0x3d4, 9);
			outportb(0x3d5, inportb(0x3d5) & 0x7f);
/*			outportb(0x3c2, inportb(0x3cc) & 0x3f);*/
			break;
	}

}